function [nextx,opt,mEi]=EGO_ask(opt)
//Ask optimizer for next data point.
//  
//Calling Sequence
//           [x,opt,mEi]=EGO_ask(opt)
//           [x,opt]=EGO_ask(opt)
//      
//
// Parameters
//            opt: mlist of type EGO.
//            
// Output
//            x: new data point, row vector.
//            opt: updated optimizer (mlist of type EGO).
//		mEI: maximum Ei found, Ei at x.
//
//Description 
//
//          Ask EGO optimizer for new data point for which criterion value is needed.
//Examples
// See EGO or demos.
//
//See also  
//              EGO
//              EGO_tell
//              EGO_stop
//              EGO_best
//              EI
//              km
//              findTheta
//              
//Authors
//          Janis Janusevskis, 3MI/EMSE
//Bibliography   
//           Jones D.R., Schonlau M. and Welch W.J. (1998), Efficient Global Optimization of Expensive Black-Box Functions, Journal of Global optimization, 13:, 455-492.
//Pujol G., Le Riche R. (2007). Description de l'architecture Scilab pour le projet RNTL/OMD.
//
    
    nextx=%nan;
    
opt.it=opt.it+1;
printf("\n\n Iteration: %d \n",opt.it);

if opt.EIoptimizer== 'RS' then // maximizes EI by Random Search
  npts=opt.EImaxpoints;
// creates candidate points
  x=grand(npts,opt.kmodel.n,'def').*(ones(npts,1).*.(opt.upb-opt.lob))+ones(npts,1).*.(opt.lob);

  // calculates EI
  Ei=EI(opt,x);
  
  //Finds max EI
  [mEi,i]=max(Ei);

nextx=x(i,:);
elseif opt.EIoptimizer=='CMA' then // maximizes EI by CMA-ES
  x=[];Ei=[];
 
		
		params=opt.EIOptparams;
   if exists('params','local') then p=params;
			else
					 npts=opt.EImaxpoints;
      // creates  CMA  optimizer
      p = cma_new();  // return (default) parameters
      x0=(opt.upb-opt.lob).*grand(1,opt.kmodel.n,'uin',1,1000)./1000+opt.lob;// generates random initial point
      p.x0 = x0';  // mandatory: initial solution vector   
      p.sigma0 = 0.3;     //?????????????????????     mandatory: initial standard deviation on x0 
      p.opt.scaling_of_variables =(opt.upb- opt.lob)';
      // how to specify lob and upb  for CMAES??

        p.stop.maxfunevals= npts;
        p.verb.plotmodulo=0; // no plotting
        p.stop.tolfun =opt.stopCrit*10^-3 ; // stop optimization if improvement in objective is less that this. Check for best value.
        p.stop.readme.tolx =0.001;// 'stop if x-changes are smaller than tolx*sigma0*scaling'; 
					end;
     
      cmaopt= cma_new(p);  // get an cma "object", constructor call
      
      while ~cma_stop(cmaopt)  
          X=cma_ask(cmaopt);
          XX=[];           
          for i = 1:length(X)
           
            // check if any of the parameters is out of bounds.
             k=0;             
             while or(X(i)<opt.lob')& or(X(i)>opt.upb') & k <100000000000
                  X(i) = cma_ask(cmaopt, 1); 
                  k=k+1;
             end;
        
        
  
            // if X still violates constraints, sets to boundary values
         //pause;
            for j=1:size(X(i),'r')
                if X(i)(j)<opt.lob(j) then
                     // disp(Par,'Warning: X violates lower constraints:'); 
                      //disp(it,'Iteration:'); 
                      //violatedConstriants=violatedConstriants+1;
                      X(i)(j)=opt.lob(j);
                      
                end;
              
                if X(i)(j)>opt.upb(j) then
                      //violatedConstriants=violatedConstriants+1;
                      //disp(Par,'Warning: X violates upper constraints:'); 
                      //disp(it,'Iteration:'); 
                      X(i)(j)=opt.upb(j);
                      
                end;
          end; // for  
          
        XX=[XX;X(i)'];// matrix length(X) rows (points), columns (dimensions)

    end;//for 
     //pause
         y=EI(opt,XX);   // calculates EI
         if opt.perf then x=[x; XX];Ei=[Ei;y]; end;
         
         cmaopt=cma_tell(cmaopt,X,-1*y');
end;// while

[mEi, nextx] = cma_best(cmaopt);
nextx=nextx';
mEi=-mEi;

//CMA optimizer
//
end;

if mEi < opt.stopCrit | opt.it>= opt.maxit then opt.stop=%t; end;



if opt.perf then
  // saves performance data
  
    opt.perfData.maxEis=[opt.perfData.maxEis; mEi];
    opt.perfData.maxEix=[opt.perfData.maxEix; nextx];
    opt.perfData.Eis($+1)=Ei;
    opt.perfData.xEis($+1)=x;
    opt.perfData.kmodels($+1)=opt.kmodel;
    
//    disp(opt.it,' It=');

printf('Maximum EI:%f\n',mEi);
end;
//pause;

endfunction

