function [R]=createCorr(X,theta,fcov)
// Creates correlation matrix R.
//
//
//Calling Sequence
//            R=createCorr(X,theta,fcov)
//
//Parameters
//            X: data points (m*n). Rows representing data points.
//            theta: parameters of correlation fuction. Row vector.
//            fcov: pointer to  correlation function.
//
// Output
//           R: correlation matrix  (m*m).
//
//Description
// Creates correlation matrix R. Used in  simGaussian and SimCondGaussian.
//
//Examples
//X=RLHS(10,3,[1 1 3],[2 4 4])
//R=createCorr(X,[1 1 1],'gaussian')
//
//See also
//           SimulateGaussian
//           SimCondGaussian
//           findTheta
//           km
//Authors
//          Janis Janusevskis, 3MI/EMSE
//Bibliography 
//
//
[m,n]=size(X);
//normalize X;

R=zeros(m,m);

					if model.cov == 'gaussian' then funct=gaussian;								
					elseif model.cov == 'matern_5_2' then funct=matern_5_2;
					elseif model.cov == 'exponential_1' then funct=exponential_1;
					elseif model.cov == 'exponential_1_9' then funct=exponential_1_9;
					end;



//for covariance functions that are not implemented in c

for i=1:m-1
  for j=i+1:m
       R(i,j)=funct(X(i,:)-X(j,:),theta);
  end;
end;
R=R+R'+eye(R);


endfunction;


